/* COPYRIGHT NOTICE
 *
 * This program comes from "The X Toolkit Cookbook" by Paul E. Kimball
 * c. 1995 Prentice Hall.
 *
 *  HDR1: Example 9-2 - events.c - Key, Button, Pointer motion event handlers
 *  
 *  HDR2: Chapter 9, page 245
 *
 * This software is provided free of charge and may be freely distributed
 * in its original form, so long as this notice appears unchanged in each 
 * copy.  This software is supplied "as-is" and is not covered by warrantee
 * of any kind, express or implied.  
 *
 * This and other examples are available by anonymous ftp from
 * ftp.netcom.com, directory "pub/pk/pkimball/cookbook".
 *
 *  1/15/95	Paul Kimball - pkimball@netcom.com
 */

/***  Include Required Header Files   ********************************/

#include <X11/Intrinsic.h>        /* The Intrinsics                  */
#include <Xm/Xm.h>                /* Motif resource symbols          */
#include <Xm/ScrolledW.h>         /* Motif XmScrolledWindow widget   */
#include <Xm/BulletinB.h>         /* Motif XmBulletinBoard widget    */
#include <Xm/PushB.h>             /* Motif XmPushButton widget       */
#include <Xm/ToggleB.h>           /* Motif XmToggleButton widget     */
#include <Xm/Text.h>              /* Motif XmText widget             */
#include <Xm/Label.h>             /* Motif XmLabel widget            */
#include <Xm/DrawingA.h>          /* Motif XmDrawingArea widget      */
#include <X11/keysym.h>           /* X Keysym definitions            */

/***** Declare Global Variables  *************************************/

Widget text_area, graphics;       /* Widget IDs needed by callbacks  */
char   output[200];               /* Output message buffer           */
void ExitCallback();              /* Callback routine to exit        */
void SetHandlers();               /* Callback - add/remove handlers  */
void KeyHandler();                /* Event handler - Keyboard input  */
void ButtonHandler();             /* Event handler - Mouse buttons   */
void MotionHandler();             /* Event handler - Pointer Motion  */

/***** Main Program  *************************************************/

main(argc,argv)
int argc;
char **argv;
{

 XtAppContext  context;                 /* Application context       */

 Widget  toplevel, manager,             /* Widget tree               */
  first_button, second_button, 
  top_label, scroll_win;

/*** Init. Toolkit, Appl. Context, Application Shell  ****************/

 toplevel = XtAppInitialize(&context, "Events", 
     NULL, 0, &argc, argv, NULL, NULL, 0);

/***  Create widget tree, bind callbacks  ****************************/

 manager = XtCreateManagedWidget("manager",
     xmBulletinBoardWidgetClass, toplevel, NULL, 0);

 top_label = XtCreateManagedWidget("topLabel",
     xmLabelWidgetClass, manager, NULL, 0);

 scroll_win = XtCreateManagedWidget("scrollWin",
     xmScrolledWindowWidgetClass, manager, NULL, 0);

 graphics = XtCreateManagedWidget("graphics",
     xmDrawingAreaWidgetClass, scroll_win, NULL, 0);

 text_area = XtCreateManagedWidget("textArea",
     xmTextWidgetClass, manager, NULL, 0);

 first_button = XtCreateManagedWidget("firstButton",
     xmToggleButtonWidgetClass, manager, NULL, 0);
 XtAddCallback(first_button,
     XmNvalueChangedCallback, SetHandlers, NULL);

 second_button = XtCreateManagedWidget("secondButton",
     xmPushButtonWidgetClass, manager, NULL, 0);
 XtAddCallback(second_button,
     XmNactivateCallback, ExitCallback, NULL);

 XtRealizeWidget(toplevel);                 /* Realize all           */
 XtAppMainLoop(context);                    /* Start main loop       */
}

/*** Callback routines and event handlers ****************************/

void ExitCallback(w, client_data, call_data)
Widget w;                                   /* Unused                */
XtPointer client_data, call_data;           /* Unused                */
{
  exit(1);
}

/*********************************************************************/

void SetHandlers(w, client_data, call_data) 
Widget w;                                   /* Unused                */
XtPointer client_data, call_data;           /* Unused                */
{

 Boolean set;

 XtVaGetValues(w, XmNset, &set, NULL);

 if (set) {
   XtAddEventHandler(graphics, KeyPressMask|KeyReleaseMask,
       False, KeyHandler, NULL);
   XtAddEventHandler(graphics, ButtonPressMask | ButtonReleaseMask,
       False, ButtonHandler, NULL);
   XtAddEventHandler(graphics, ButtonMotionMask,
       False, MotionHandler, NULL); 
 } else {
   XtRemoveEventHandler(graphics, KeyPressMask|KeyReleaseMask, 
       False, KeyHandler, NULL);
   XtRemoveEventHandler(graphics, ButtonPressMask|ButtonReleaseMask,
       False, ButtonHandler, NULL);
   XtRemoveEventHandler(graphics, ButtonMotionMask, 
       False, MotionHandler, NULL);
 }
}


void KeyHandler(w, client_data, event, continue_dispatch)
Widget    w;                                /* unused                */
XtPointer client_data;                      /* unused                */
XEvent    *event;                           /* Key event pointer     */
Boolean   *continue_dispatch;               /* unused                */
{

  KeySym keysym;
  char   buffer[5];
  int    length;
  Modifiers dummy;

  output[0] = (char)NULL;                   /* Initialize output     */

  if ( event->type == KeyPress )            /* Print out message     */
      sprintf(&output[strlen(output)], "KeyPress Detected:\n");
  if ( event->type == KeyRelease ) 
      sprintf(&output[strlen(output)], "KeyRelease Detected:\n");

  sprintf(&output[strlen(output)],          /* Print out keycode     */
      "  KeyCode = %x\n", event->xkey.keycode);

  XtTranslateKeycode(XtDisplay(w),          /* Convert to keysym     */
      event->xkey.keycode, event->xkey.state, &dummy, &keysym);

  switch(keysym){                           /* Detect special input  */
      case XK_c:                            /* Ctrl-c means "exit"   */
      case XK_C:
          if (event->xkey.state & ControlMask) exit(1);
      break;
  }

  sprintf(&output[strlen(output)],
      "  KeySym = %x (%s)\n", keysym, XKeysymToString(keysym));

  length = XLookupString((XKeyEvent*)event, /* Convert to ASCII      */
       buffer, 4, &keysym, (XComposeStatus*)NULL); 
  buffer[length] = NULL;                    /* Terminate ASCII string*/
  sprintf(&output[strlen(output)],
   "  ascii = '%s'\n", buffer);             /* Print ASCII value     */

  XtVaSetValues(text_area, XmNvalue, output, NULL);

}

/*********************************************************************/

void ButtonHandler(w, client_data, event, continue_dispatch)
Widget    w;                                /* widget ID             */
XtPointer client_data;                      /* unused                */
XEvent    *event;                           /* event pointer         */
Boolean   *continue_dispatch;               /* unused                */
{

 int button;
 output[0] = (char)NULL;                    /* Initialize output     */
 button = event->xbutton.button;            /* Get button ID         */

/****   Print out message   ******************************************/

 if (event->xbutton.type == ButtonPress){ 
    sprintf(&output[strlen(output)], "ButtonPress Detected:\n");

  /* User clicked in graphics area, so put the keyboard focus there  */
  /* Every widget set expects to do this a bit differently.          */
  /* See Section 9.8 for details                                     */

  /* This works, but breaks overall traversal in Motif:              */
  /*   XtSetKeyboardFocus(toplevel, w);                              */

  /* This does not work - Motif widgets do not export accept_focus:  */
  /*   XtCallAcceptFocus(w, event->xbutton.time );                   */

  /* The Motif-specific routine works properly:                      */
    XmProcessTraversal(w, XmTRAVERSE_CURRENT); 

 }
 else if ( event->xbutton.type == ButtonRelease ) {
    sprintf(&output[strlen(output)], "ButtonRelease Detected:\n");
 }

 sprintf(&output[strlen(output)],           /* Print out button id   */
   "  Button number = %i\n", button);

 XtVaSetValues(text_area, XmNvalue, output, NULL);

}

/*********************************************************************/

void MotionHandler(w, client_data, event, continue_dispatch)
Widget    w;                                /* unused                */
XtPointer client_data;                      /* unused                */
XEvent    *event;                           /* event pointer         */
Boolean   *continue_dispatch;               /* unused                */
{

  Position x, y, rootx, rooty;
  output[0] = NULL;                         /* Initialize output     */

  sprintf(&output[strlen(output)],          /* Print a message       */
       "PointerMotion Detected:\n");

  x = event->xmotion.x;                     /* Get new coordinates   */
  y = event->xmotion.y;                     /* in event window       */

  rootx = event->xmotion.x_root;            /* Get new coordinates   */
  rooty = event->xmotion.y_root;            /* in root window        */

  sprintf(&output[strlen(output)],          /* Print out location    */
       "  New coords (window) = %d x, %d y\n", x, y);
  sprintf(&output[strlen(output)],
       "  New coords (root) = %d x, %d y\n", rootx, rooty);

  XtVaSetValues(text_area, XmNvalue, output, NULL);

}
