/* COPYRIGHT NOTICE
 *
 * This program comes from "The X Toolkit Cookbook" by Paul E. Kimball
 * c. 1995 Prentice Hall.
 *
 *  HDR1: Example 5-3 - paintit.c- Explicit color manipulation
 *                           Also includes resource allocation routines
 *                           from Chapter 5
 *  
 *  HDR2: Chapter 5, page 95
 *
 * This software is provided free of charge and may be freely distributed
 * in its original form, so long as this notice appears unchanged in each 
 * copy.  This software is supplied "as-is" and is not covered by warrantee
 * of any kind, express or implied.  
 *
 * This and other examples are available by anonymous ftp from
 * ftp.netcom.com, directory "pub/pk/pkimball/cookbook".
 *
 *  1/15/95	Paul Kimball - pkimball@netcom.com
 */

/***  Include required header files ********************************/

#include <X11/Intrinsic.h>       /* The Intrinsics                 */
#include <X11/StringDefs.h>      /* Intrinsics "XtN" symbols       */
#include <Xm/Xm.h>               /* Motif "Xm" types, symbols      */
#include <Xm/BulletinB.h>        /* Motif XmBulletinBoard widget   */
#include <Xm/Label.h>            /* Motif XmLabel widget           */
#include <Xm/PushB.h>            /* Motif XmPushButton widget      */
#include <Xm/Text.h>             /* Motif XmText widget            */

/***** Declare global variables  ***********************************/

XtAppContext  context;                  /* Application context     */
Widget     toplevel, manager,           /* Widget tree             */
           top_label, text_area,
           first_button, second_button;

Pixel      fg, bg;                                       /* Colors */

XmString   xmstring;                    /* Motif compound string   */
XmFontList fontlist;                    /* Motif font list         */
String     courier12 =                  /* An XLFD Font Name       */
           "-*-Courier-*-R-Normal--*-120-*-*-*-*-ISO8859-1";

/***** Forward declare callback routines and utilities *************/

Pixel      GetNamedColor();             /* Allocates Pixel value   */
Pixel      GetRgbColor();               /* Allocates Pixel value   */
void       FreeColor();                 /* Frees pixel value       */
XmFontList GetXmFontList();             /* Loads font for Motif    */

void       ExitCallback();              /* Still just exits        */
void       Paintit();                   /* Paints widgets          */

/***** Main program  ***********************************************/

main(argc,argv)
int argc;
char **argv;
{

/***  Init. toolkit, app. context, application shell  **************/

 toplevel = XtVaAppInitialize (          /* Create application shell  */
     &context, "Paintit",                /* Application name, class   */
     NULL, 0,                            /* options, num_options      */
     &argc, argv,                        /* Command line              */
     NULL,                               /* Fallback resources        */
     XmNtitle, "Resource Example",       /* Initial resources         */
     NULL );

/* Allocate server resources for initial resource settings */

 fontlist = GetXmFontList( toplevel, courier12 );

 fg = XBlackPixelOfScreen( XtScreen(toplevel) );
 bg = GetRgbColor( toplevel, 55000, 55000, 55000 );

/*****  Create widget tree  *******************************************/

 manager = XtVaCreateManagedWidget(       /* XmBulletinBoard          */
     "manager", xmBulletinBoardWidgetClass,
     toplevel,
     XmNbackground, bg,                   /* Set background color     */
     NULL);

 xmstring = XmStringCreateLocalized( "Enter a Color Name:");
 
 top_label = XtVaCreateManagedWidget(     /* Create XmLabel           */
     "topLabel", xmLabelWidgetClass,      /* name and class pointer   */
     manager,                             /* parent                   */
     XmNx, 10,                            /* Set X position           */
     XmNy, 10,                            /* Set Y position           */
     XmNforeground, fg,                   /* Set foreground color     */
     XmNbackground, bg,                   /* Set background color     */
     XmNlabelString, xmstring,            /* Set displayed XmString   */
     XmNfontList, fontlist,               /* Set the font             */
     NULL );

 XmStringFree(xmstring);                  /* Free XmString storage    */

 text_area = XtVaCreateManagedWidget(     /* Create XmText            */
     "textArea", xmTextWidgetClass,       /* name and class pointer   */
     manager,                             /* parent                   */
     XmNx, 10,                            /* Set X position           */
     XmNy, 40,                            /* Set Y position           */
     XmNforeground, fg,                   /* Set foreground color     */
     XmNbackground, bg,                   /* Set background color     */
     XmNvalue, "Type text here",          /* Set displayed String     */
     XmNfontList, fontlist,               /* Set the font             */
     NULL );

 xmstring = XmStringCreateLocalized("Paint All");

 first_button = XtVaCreateManagedWidget(  /* Create XmPushButton      */
     "firstButton", xmPushButtonWidgetClass, 
     manager,
     XmNx, 10,                            /* Set X position           */
     XmNy, 90,                            /* Set Y position           */
     XmNwidth, 100,                       /* Set width                */
     XmNforeground, fg,                   /* Set foreground color     */
     XmNbackground, bg,                   /* Set background color     */
     XmNlabelString, xmstring,            /* Set displayed XmString   */
     XmNfontList, fontlist,               /* Set the font             */
     NULL );

 XmStringFree(xmstring);                  /* Free XmString storage    */
     xmstring = XmStringCreateLocalized( "Exit");

 second_button = XtVaCreateManagedWidget( /* Create XmPushButton      */
     "secondButton", xmPushButtonWidgetClass, 
     manager,
     XmNx, 120,                           /* Set X position           */
     XmNy, 90,                            /* Set Y position           */
     XmNwidth, 100,                       /* Set width                */
     XmNforeground, fg,                   /* Set foreground color     */
     XmNbackground, bg,                   /* Set background color     */
     XmNlabelString, xmstring,            /* Set displayed XmString   */
     XmNfontList, fontlist,               /* Set the font             */
     NULL );
 XmStringFree(xmstring);                  /* Free XmString storage    */

/***** Register callback routines *************************************/

 XtAddCallback(text_area,                 /* Add callback to text area */
     XmNactivateCallback,                 /* Call back when CR pressed */ 
     Paintit, NULL);                      /* Routine to call           */

 XtAddCallback(first_button,              /* Add callback to button    */
     XmNactivateCallback,                 /* Call back when "pressed"  */
     Paintit, NULL);                      /* Routine to call           */

 XtAddCallback(second_button,             /* Add callback to button    */
     XmNactivateCallback,                 /* Call back when "pressed"  */
     ExitCallback, NULL);                 /* Routine to call           */

/***** Realize widget tree and start main loop *************************/

 XtRealizeWidget(toplevel);               /* Realize entire tree       */
 XtAppMainLoop(context);                  /* Can't get past here       */
}

/***** Callback Routines ***********************************************/

void ExitCallback(w, client_data, call_data)
Widget  w;
XtPointer client_data;
XtPointer call_data;
{
  exit();
}


void Paintit(w, client_data, call_data)
Widget  w;
XtPointer client_data;
XtPointer call_data;
{
  String  input_name;
  Pixel  new_color, old_color;
  Widget  *children;
  int    num_children;
  int    n;

/* Get color name and previous background color from text widget */

  XtVaGetValues(text_area,
      XmNvalue, &input_name,
      XmNbackground, &old_color,
      NULL);

/* Get a new pixel value from the server and free the old one,  */


/* But don't paint background black, since it's also foreground */

  new_color = GetNamedColor(w, input_name); 
  if ( !new_color || new_color == XBlackPixelOfScreen(XtScreen(w)) ) {
      XtVaSetValues(text_area, XmNvalue, "Can't use that color", NULL);
      return;
  }
  FreeColor(w, old_color);

  /* Set the new color in the parent */
  /* Then get list of children and set new color in them, too */

  XtVaSetValues(manager, XmNbackground, new_color, NULL );

  XtVaGetValues(manager, 
      XmNnumChildren, &num_children,
      XmNchildren, &children,
      NULL );

  for(n=0; n<num_children; n++){
      XtVaSetValues(children[n], XmNbackground, new_color, NULL );
  }
}



/* GetRgbColor()   - Code Fragment 5-10  */

Pixel
GetRgbColor(widget, red, green, blue)
Widget          widget;                          /* A widget ID        */
unsigned short  red, green, blue;                /* Desired RGB values */
{

 Display   *display;
 Colormap  cmap;
 XColor   alloc_color;

 display = XtDisplay(widget);                    /* Get display pointer   */

 XtVaGetValues(widget, XtNcolormap, &cmap, NULL );

 alloc_color.red = red;                          /* Red value, 0 - 65535  */
 alloc_color.green =  green;                     /* Green value, 0 - 65535*/
 alloc_color.blue = blue;                        /* Blue value, 0 - 65535 */
 alloc_color.flags =  DoRed|DoGreen|DoBlue;      /* Set all */

 if (XAllocColor(display, cmap, &alloc_color)){  /* Success */
    return alloc_color.pixel;
 }
 else {                /* Failed  */
    printf("Can't allocate color!\n");     /* Print warning */
    return NULL;           /* Return NULL   */
 }
}


/* GetNamedColor() - Code Fragment 5-11  */

Pixel 
GetNamedColor(widget, colorname)
Widget  widget;
String  colorname;
{
  Display  *display;
  Colormap  cmap;
  XColor   alloc_color, exact_color;

  display = XtDisplay(widget);    /* Get display pointer   */

  XtVaGetValues(widget, XtNcolormap, &cmap, NULL );

  if ( XAllocNamedColor(
         display,                 /* Server connection                     */
         cmap,                    /* Colormap                              */
         colorname,               /* color name - see /usr/lib/X11/rgb.txt */
         &alloc_color,            /* returned RGB and pixel as allocated   */
         &exact_color)){          /* RGB as defined in server's database   */
     return alloc_color.pixel;
  }
  else {                                   /* Failed */
     printf("Can't allocate color!\n");    /* Print warning */
     return NULL;
  }
}



/* FreeColor()     - Code Fragment 5-13  */

void
FreeColor(widget, pixel_value)
Widget  widget;
Pixel   pixel_value;
{
  Colormap  cmap;
  XtVaGetValues(widget, XtNcolormap, &cmap, NULL );
  XFreeColors(
      XtDisplay(widget),  /* Display                            */
      cmap,               /* Colormap in which they are defined */
      &pixel_value,       /* Array of pixel values              */
      1,                  /* Only 1 in the array                */
      0);                 /* No planes to free                  */
}



/* GetXmFontList() - See Code Fragment 5-18  */

XmFontList 
GetXmFontList(widget, fontname)
Widget widget;
String fontname;
{
  Display         *display;
  XFontStruct     *fontstruct;
  XmFontListEntry entry;
  XmFontList      fontlist;

  display = XtDisplay(widget);   /* Get widget display pointer */

  if ( !(fontstruct = XLoadQueryFont(display, fontname) ) ){
      printf("Can't allocate font!!!\n"); 
      return (XmFontList)NULL;
  }
  else {
    if ( XmVersion >= 1002 ) {
      entry = XmFontListEntryCreate(    /* Motif 1.2 method     */
        XmFONTLIST_DEFAULT_TAG,         /* Will be default font */
        XmFONT_IS_FONT,                 /* Single font          */
        fontstruct);                    /* XFontStruct pointer  */
      fontlist = XmFontListAppendEntry(
        NULL,                           /* This is new font list*/
        entry);                         /* Entry to append      */
    }
    else {
      fontlist = XmFontListCreate(      /* Motif 1.1 method     */
        fontlist,
        XmSTRING_DEFAULT_CHARSET);
    }
    return fontlist;
  }
}
