/* COPYRIGHT NOTICE
 *
 * This program comes from "The X Toolkit Cookbook" by Paul E. Kimball
 * c. 1995 Prentice Hall.
 *
 *  HDR1: Example 11-2 winmgr.c - Pop-ups, Shell resources and window managers
 *  
 *  HDR2: Chapter 11, page 334
 *
 * This software is provided free of charge and may be freely distributed
 * in its original form, so long as this notice appears unchanged in each 
 * copy.  This software is supplied "as-is" and is not covered by warrantee
 * of any kind, express or implied.  
 *
 * This and other examples are available by anonymous ftp from
 * ftp.netcom.com, directory "pub/pk/pkimball/cookbook".
 *
 *  1/15/95	Paul Kimball - pkimball@netcom.com
 */

 /* winmgr.c */

/***  Include Required Header Files   ********************************/

#include <X11/Intrinsic.h>                  /* The Intrinsics        */
#include <X11/StringDefs.h>                 /* Resource symbols      */
#include <X11/Shell.h>                      /* Shell widgets         */
#include <Xm/PushB.h>                       /* Motif XmPushButton    */
#include <Xm/RowColumn.h>                   /* Motif XmRowColumn     */
#include "app_icon"                         /* Bitmap from Chap 5    */
             
/***  Declare Global Variables   *************************************/

Display       *display;                     /* Server connection     */
XtAppContext  context;                      /* Application context   */
Widget        toplevel, manager,            /* Widget tree           */
              popup_shell, popdown_btn,
              exit_btn, popup_btn, iconify_btn;
Boolean       shell_is_popped = False;      /* True if popup is up   */
Pixmap        app_icon_pixmap;              /* Icon bitmap           */
int           state;                        /* Current icon state    */

/***  Forward Declare Callback Routines   ****************************/

void ExitCallback();          /* Callback: exit                      */
void Popup();                 /* Callback: pop up window             */
void Popdown();               /* Callback: pop down popup window     */
void AdjustPopup();           /* Callback: position popup window     */
void Iconify();               /* Callback: iconify window            */
void Deiconify();             /* Timer: deiconify after 5 seconds    */
void IconStateHandler();      /* Event handler: detect state change  */

/***  Main Program  **************************************************/

main(argc,argv)
int argc;
char **argv;
{

 toplevel = XtAppInitialize(&context, "Winmgr", 
     NULL, 0, &argc, argv, NULL, NULL, 0 );

 app_icon_pixmap = XCreateBitmapFromData (  /* Create 1-plane Pixmap */
     XtDisplay(toplevel),                   /*  to use as icon       */
     XRootWindowOfScreen(XtScreen(toplevel)), 
     app_icon_bits, app_icon_width, app_icon_height);

 XtVaSetValues(toplevel, XmNiconPixmap, app_icon_pixmap, NULL );

 XtAddEventHandler(toplevel,             /* event handler traps Map  */
     StructureNotifyMask, False,         /* and UnmapNotify events   */
     IconStateHandler, NULL);            /* signaling icon state     */

/***  Create widget tree, bind callbacks  ****************************/

 manager = XtCreateManagedWidget( "manager",       /* Manager        */
     xmRowColumnWidgetClass, toplevel, NULL, 0);
         
 popup_btn = XtCreateManagedWidget("popupBtn",     /* "Popup" button */
     xmPushButtonWidgetClass, manager, NULL, 0);
 XtAddCallback( popup_btn, XmNactivateCallback, Popup, NULL);

 iconify_btn = XtCreateManagedWidget("iconifyBtn", /* "Iconify" button*/
     xmPushButtonWidgetClass, manager, NULL, 0);
 XtAddCallback( iconify_btn, XmNactivateCallback, Iconify, NULL);

 exit_btn = XtCreateManagedWidget("exitBtn",       /* "Exit" button  */
     xmPushButtonWidgetClass, manager, NULL, 0);
 XtAddCallback( exit_btn, XmNactivateCallback, ExitCallback, NULL );

 popup_shell = XtCreatePopupShell("popupShell",    /* Pop-up shell   */
     transientShellWidgetClass, toplevel, NULL, 0);
 XtAddCallback(popup_shell, XmNpopupCallback, AdjustPopup, NULL);

 popdown_btn = XtCreateManagedWidget("popdownBtn", /* The pop-up child*/
     xmPushButtonWidgetClass, popup_shell, NULL, 0);
 XtAddCallback(popdown_btn, XmNactivateCallback, Popdown, NULL);
 
 XtRealizeWidget(toplevel);                  /* Realize main window  */

 XtVaSetValues(popup_shell,                  /* Set popup resources  */
     XmNwindowGroup, XtWindow(toplevel),     /* App Shell is leader  */
     XmNtransientFor, toplevel,              /* Limit decoration     */
     XmNdeleteResponse, XmUNMAP,             /* Unmap if wm says to  */
     NULL);
     
 XtAppMainLoop(context);                     /* Start Event Loop     */
}

/*** Application Callback Routines   *********************************/

void                                        /* Pop up secondary shell*/
Popup(w, client_data, call_data)
Widget w;                                   /* Unused                */
XtPointer client_data, call_data;           /* Unused                */
{
   XtPopup(popup_shell, XtGrabNone);        /* Pop without grab      */
}

/*********************************************************************/

void                                        /* Pop down the shell    */
Popdown(w, client_data, call_data)
Widget w;                                   /* Unused                */
XtPointer client_data, call_data;           /* Unused                */
{
   XtPopdown(popup_shell);                  /* Pop it down           */
}

/*********************************************************************/

void                                        /* Place popup shell in  */
AdjustPopup(w, client_data, call_data)      /* middle of main window */
Widget w;                                   /* Shell to be centered  */
XtPointer client_data, call_data;           /* Unused                */
{

 Position  root_x, root_y;                  /* Position - root coords*/
 Dimension toplevel_wid, toplevel_hgt;      /* Size of Appl Shell    */
 Dimension popup_wid, popup_hgt;            /* Size of pop-up        */
 Position  x_offset, y_offset;              /* Offset from Appl Shell*/

 XtVaGetValues(toplevel,                    /* Find appl. shell size */
     XmNwidth, &toplevel_wid, 
     XmNheight, &toplevel_hgt, 
     NULL );
 XtVaGetValues(w,                           /* Find popup shell size */
     XmNwidth, &popup_wid,
     XmNheight, &popup_hgt, 
     NULL);
     
 x_offset = (toplevel_wid - popup_wid)/2;   /* Calculate popup offset*/
 y_offset = (toplevel_hgt - popup_hgt)/2;   /* from appl shell origin*/

 XtTranslateCoords ( toplevel,              /* Convert this to root  */
     x_offset, y_offset, &root_x, &root_y); /* window coordinates    */

 XtVaSetValues(w,                           /* Set popup shell X & Y */
     XmNx, root_x, 
     XmNy, root_y, 
     NULL);
}

/*********************************************************************/

void                                          /* Iconify application   */
Iconify(w, client_data, call_data)            /* under program control */
Widget w;                                     /* Unused                */
XtPointer client_data, call_data;             /* Unused                */
{
 /* If not iconified, set XmNiconic to change state */
 XtVaSetValues(toplevel, XmNiconic, True, NULL);

 /* If that doesn't work, you have an old broken toolkit              */
 /* try using the R4 Xlib routine:                                    */
 /* XIconifyWindow(XtDisplay(toplevel), XtWindow(toplevel), 0);       */

 /* Set timer to deiconify*/
 XtAppAddTimeOut(context, 5000, Deiconify, NULL);
}

/***********************************************************************/

void                                          /* Timer callback to     */
Deiconify(client_data, timer_id)              /*  deiconify windows    */
XtPointer client_data;                        /* Unused                */
XtIntervalId *timer_id;                       /* Unused                */
{

 /* If currently iconified, set XmNiconic to change state */
 if (state==IconicState)
   XtVaSetValues(toplevel, XmNiconic, False, NULL); 

 /* If that doesn't work, you have an old broken toolkit             */
 /* try mapping shell with Xlib routine:                             */
 /* XMapWindow(XtDisplay(toplevel), XtWindow(toplevel));             */

}

/*********************************************************************/

void                                        /* Detect state change   */
IconStateHandler(w, client_data, event, dispatch)
Widget w;                                   /* Widget ID of Shell    */
XtPointer client_data;                      /* unused                */
XEvent *event;                              /* Map or Unmap event    */
Boolean dispatch;                           /* unused                */
{
 switch (event->type){                      /* switch on event type  */
   case UnmapNotify:                        /* Shell is iconified    */
     state = IconicState;                   /* Set state variable    */
     XBell(XtDisplay(w), 50);               /* Ring bell to signal   */
     break;
   case MapNotify:                          /* Shell is deiconified  */
     state = NormalState;                   /* Set state variable    */
     XBell(XtDisplay(w), 50);               /* Ring bell to signal   */
     break;
 }
}

/*********************************************************************/

void                                        /* Exit the application  */
ExitCallback(w, client_data, call_data)
Widget w;                                    /* unused */
XtPointer client_data, call_data;            /* unused */
{
 exit(1);                                    /* Exit */
}
